"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.collapse = collapse;

var _TBlock = require("../tree/TBlock");

var _TPhrasing = require("../tree/TPhrasing");

var _TText = require("../tree/TText");

var _textTransforms = require("./text-transforms");

function collapseText(node) {
  if (node.hasWhiteSpaceCollapsingEnabled) {
    node.data = (0, _textTransforms.normalizeWhitespaces)((0, _textTransforms.replaceSegmentBreaks)((0, _textTransforms.normalizeSpaceDiscardingCharset)((0, _textTransforms.normalizeZeroWidthWhitespaces)((0, _textTransforms.removeConsecutiveSegmentBreaks)((0, _textTransforms.removeCollapsibleAroundSegmentBreak)(node.data))))));
  }

  return node;
}

function collapseBlock(node) {
  const newChildren = [];

  for (const i in node.children) {
    const child = collapseNode(node.children[i]);

    if (!child.isWhitespace()) {
      child.trimLeft();
      child.trimRight();

      if (!child.isEmpty()) {
        newChildren.push(child);
      }
    }
  }

  node.bindChildren(newChildren);
  return node;
}

function collapsePhrasing(node) {
  let collapsedChildren = [];
  let trimmedChildren = [];

  for (const i in node.children) {
    const child = collapseNode(node.children[i]);

    if (child instanceof _TText.TText) {
      if (!child.isEmpty()) {
        collapsedChildren.push(child);
      }
    } else if (child instanceof _TPhrasing.TPhrasing) {
      if (!child.isEmpty()) {
        collapsedChildren.push(child);
      }
    }
  }

  collapsedChildren.forEach((childI, i) => {
    const k = i + 1;

    if (k < node.children.length - 1) {
      const childK = node.children[k];

      if (childI.isCollapsibleRight() && childK.isCollapsibleLeft()) {
        childI.trimRight();

        if (!childI.isEmpty()) {
          trimmedChildren.push(childI);
        }
      } else {
        trimmedChildren.push(childI);
      }
    } else {
      trimmedChildren.push(childI);
    }
  });
  node.bindChildren(trimmedChildren);
  return node;
}

function collapseNode(node) {
  if (node instanceof _TText.TText) {
    return collapseText(node);
  }

  if (node instanceof _TBlock.TBlock) {
    return collapseBlock(node);
  }

  if (node instanceof _TPhrasing.TPhrasing) {
    return collapsePhrasing(node);
  }

  return node;
}

function collapse(tree) {
  const root = collapseNode(tree);
  root.trimLeft();
  root.trimRight();
  return root;
}
//# sourceMappingURL=collapse.js.map