"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.hoist = hoist;

var _TBlock = require("../tree/TBlock");

var _TBlockAnchor = require("../tree/TBlockAnchor");

var _TPhrasing = require("../tree/TPhrasing");

var _TPhrasingAnchor = require("../tree/TPhrasingAnchor");

var _TText = require("../tree/TText");

function groupText(tnode, wrappernode) {
  let newChildren = [];
  let wrapper = wrappernode.newEmpty();

  for (const child of tnode.children) {
    const newChild = child;

    if (newChild instanceof _TText.TText || newChild instanceof _TPhrasing.TPhrasing) {
      wrapper.children.push(newChild);
    } else {
      if (wrapper.children.length) {
        newChildren.push(wrapper);
        wrapper = wrappernode.newEmpty();
      }

      if (wrappernode instanceof _TPhrasingAnchor.TPhrasingAnchor) {
        const nextChild = new _TBlockAnchor.TBlockAnchor({ ...newChild.cloneInitParams(),
          href: wrappernode.href,
          parentStyles: wrappernode.parentStyles
        });
        newChildren.push(nextChild);
      } else {
        newChildren.push(newChild);
      }
    }
  }

  if (wrapper.children.length) {
    newChildren.push(wrapper);
  }

  tnode.bindChildren(newChildren);
  return tnode;
}

function hoistNode(node) {
  node.bindChildren(node.children.map(hoistNode));

  if (node instanceof _TPhrasing.TPhrasing) {
    for (const child of node.children) {
      if (child instanceof _TBlock.TBlock) {
        const newNode = new _TBlock.TBlock(node.cloneInitParams({
          parentStyles: child.parentStyles,
          styles: child.styles
        }));
        newNode.bindChildren(node.children);
        return groupText(newNode, node);
      }
    }
  } else if (node instanceof _TBlock.TBlock) {
    if (node.children.length > 0) {
      return groupText(node, new _TPhrasing.TPhrasing({
        parentStyles: node.styles,
        stylesMerger: node.stylesMerger
      }));
    }
  }

  return node;
}

function hoist(tree) {
  return hoistNode(tree);
}
//# sourceMappingURL=hoist.js.map