"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TStylesMerger = exports.emptyProcessedPropsReg = void 0;

var _cssProcessor = _interopRequireWildcard(require("@native-html/css-processor"));

var _elementsModel = require("../model/elements-model");

var _TStyles = require("./TStyles");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const emptyProcessedPropsReg = new _cssProcessor.CSSProcessedProps();
exports.emptyProcessedPropsReg = emptyProcessedPropsReg;

function mapMixedStyleRecordToCSSProcessedPropsReg(processor, styles) {
  let regStyles = {};

  for (const key in styles) {
    regStyles[key] = processor.compileStyleDeclaration(styles[key]);
  }

  return regStyles;
} // Specificity hierarchy, in descending order:
// 1. Inline styles
// 2. ID (idsStyles)
// 3. Classes (classesStyles)
// 4. Element name (tagsStyles)
// 5. Attribute styles (styles derived from attributes)
// 6. Default element styles
// 7. Inherited styles (baseFontStyles)


class TStylesMerger {
  constructor(config, cssProcessorConfig) {
    _defineProperty(this, "processor", void 0);

    _defineProperty(this, "tagsStyles", void 0);

    _defineProperty(this, "classesStyles", void 0);

    _defineProperty(this, "idsStyles", void 0);

    _defineProperty(this, "enableCSSInlineProcessing", void 0);

    _defineProperty(this, "enableUserAgentStyles", void 0);

    this.processor = new _cssProcessor.default(cssProcessorConfig);
    this.classesStyles = mapMixedStyleRecordToCSSProcessedPropsReg(this.processor, config.classesStyles);
    this.tagsStyles = mapMixedStyleRecordToCSSProcessedPropsReg(this.processor, config.tagsStyles);
    this.idsStyles = mapMixedStyleRecordToCSSProcessedPropsReg(this.processor, config.idsStyles);
    this.enableCSSInlineProcessing = config.enableCSSInlineProcessing;
    this.enableUserAgentStyles = config.enableUserAgentStyles;
  }

  compileInlineCSS(inlineCSS) {
    return this.processor.compileInlineCSS(inlineCSS);
  }

  compileStyleDeclaration(styleDeclaration) {
    return this.processor.compileStyleDeclaration(styleDeclaration);
  }

  buildStyles(inlineStyle, parentStyles, descriptor) {
    var _this$tagsStyles, _this$idsStyles, _model$defaultCSSProc;

    const ownInlinePropsReg = this.enableCSSInlineProcessing && inlineStyle ? this.compileInlineCSS(inlineStyle) : null;
    const model = descriptor.tagName ? (0, _elementsModel.getElementModelFromTagName)(descriptor.tagName) : null;
    const userTagOwnProps = (_this$tagsStyles = this.tagsStyles[descriptor.tagName]) !== null && _this$tagsStyles !== void 0 ? _this$tagsStyles : null;
    const userIdOwnProps = (_this$idsStyles = this.idsStyles[descriptor.id]) !== null && _this$idsStyles !== void 0 ? _this$idsStyles : null;
    const classes = descriptor.className ? descriptor.className.split(/\s+/) : [];
    const userClassesOwnPropsList = classes.map(c => this.classesStyles[c] || null);
    const derivedPropsFromAttributes = this.enableUserAgentStyles ? (model === null || model === void 0 ? void 0 : model.getUADerivedCSSProcessedPropsFromAttributes(descriptor.attributes)) || null : null;
    const userAgentTagProps = this.enableUserAgentStyles ? (_model$defaultCSSProc = model === null || model === void 0 ? void 0 : model.defaultCSSProcessedProps) !== null && _model$defaultCSSProc !== void 0 ? _model$defaultCSSProc : null : null; // Latest properties will override former properties.

    const mergedOwnProps = mergePropsRegistries([userAgentTagProps, derivedPropsFromAttributes, userTagOwnProps, ...userClassesOwnPropsList, userIdOwnProps, ownInlinePropsReg]);
    return new _TStyles.TStyles(mergedOwnProps, parentStyles);
  }

}
/**
 * Merge from lower specificy registries (left) to higher specificity registry (right).
 * @param registries
 */


exports.TStylesMerger = TStylesMerger;

function mergePropsRegistries(registries) {
  const filteredProps = registries.filter(isNotNull);
  return emptyProcessedPropsReg.merge(...filteredProps);
}

function isNotNull(item) {
  return item !== null;
}
//# sourceMappingURL=TStylesMerger.js.map